#!/usr/bin/env python3
"""
Async MQTT gRPC Client
High-performance async MQTT client using grpc.aio

Performance Benefits:
- True async I/O for IoT messaging
- High-throughput concurrent publishing
- Non-blocking streaming subscriptions
- Efficient device management
"""

import asyncio
import logging
from typing import List, Dict, Optional, Any, AsyncIterator, Callable, TYPE_CHECKING
from google.protobuf.struct_pb2 import Struct
from .async_base_client import AsyncBaseGRPCClient
from .proto import mqtt_service_pb2, mqtt_service_pb2_grpc

if TYPE_CHECKING:
    from .consul_client import ConsulRegistry

logger = logging.getLogger(__name__)


class AsyncMQTTClient(AsyncBaseGRPCClient):
    """Async MQTT gRPC client for high-throughput IoT messaging."""

    def __init__(
        self,
        host: Optional[str] = None,
        port: Optional[int] = None,
        user_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        lazy_connect: bool = True,
        enable_compression: bool = True,
        enable_retry: bool = True,
        consul_registry: Optional['ConsulRegistry'] = None,
        service_name_override: Optional[str] = None
    ):
        """
        Initialize async MQTT client.

        Args:
            host: Service address (optional)
            port: Service port (optional)
            user_id: User ID
            organization_id: Organization ID
            lazy_connect: Lazy connection (default: True)
            enable_compression: Enable compression (default: True)
            enable_retry: Enable retry (default: True)
            consul_registry: ConsulRegistry instance for service discovery
            service_name_override: Override service name for Consul lookup
        """
        super().__init__(
            host=host,
            port=port,
            user_id=user_id,
            lazy_connect=lazy_connect,
            enable_compression=enable_compression,
            enable_retry=enable_retry,
            consul_registry=consul_registry,
            service_name_override=service_name_override
        )
        self.organization_id = organization_id or 'default-org'

    def _create_stub(self):
        """Create MQTT service stub."""
        return mqtt_service_pb2_grpc.MQTTServiceStub(self.channel)

    def service_name(self) -> str:
        return "MQTT"

    def default_port(self) -> int:
        return 50053

    # ============================================
    # Health Check
    # ============================================

    async def health_check(self, deep_check: bool = False) -> Optional[Dict]:
        """Health check."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.MQTTHealthCheckRequest(deep_check=deep_check)
            response = await self.stub.HealthCheck(request)

            return {
                'healthy': response.healthy,
                'broker_status': response.broker_status,
                'active_connections': response.active_connections,
                'message': response.message
            }

        except Exception as e:
            return self.handle_error(e, "health check")

    # ============================================
    # Connection Management
    # ============================================

    async def mqtt_connect(self, client_id: str, username: str = '',
                          password: str = '') -> Optional[Dict]:
        """Connect to MQTT service."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.ConnectRequest(
                client_id=client_id,
                user_id=self.user_id,
                username=username,
                password=password
            )

            response = await self.stub.Connect(request)

            if response.success:
                return {
                    'success': True,
                    'session_id': response.session_id,
                    'message': response.message
                }
            return None

        except Exception as e:
            return self.handle_error(e, "connect")

    async def disconnect(self, session_id: str) -> Optional[Dict]:
        """Disconnect from MQTT service."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.DisconnectRequest(
                session_id=session_id,
                user_id=self.user_id
            )

            response = await self.stub.Disconnect(request)

            if response.success:
                return {'success': True, 'message': response.message}
            return None

        except Exception as e:
            return self.handle_error(e, "disconnect")

    async def get_connection_status(self, session_id: str) -> Optional[Dict]:
        """Get connection status."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.ConnectionStatusRequest(
                session_id=session_id,
                user_id=self.user_id
            )

            response = await self.stub.GetConnectionStatus(request)

            return {
                'connected': response.connected,
                'connected_at': response.connected_at,
                'messages_sent': response.messages_sent,
                'messages_received': response.messages_received
            }

        except Exception as e:
            return self.handle_error(e, "get connection status")

    # ============================================
    # Publishing
    # ============================================

    async def publish(self, session_id: str, topic: str, payload: bytes,
                     qos: int = 1, retained: bool = False) -> Optional[Dict]:
        """Publish message."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.PublishRequest(
                user_id=self.user_id,
                session_id=session_id,
                topic=topic,
                payload=payload,
                qos=qos,
                retained=retained
            )

            response = await self.stub.Publish(request)

            if response.success:
                return {'success': True, 'message_id': response.message_id}
            return None

        except Exception as e:
            return self.handle_error(e, "publish")

    async def publish_batch(self, session_id: str,
                           messages: List[Dict]) -> Optional[Dict]:
        """
        Publish multiple messages in batch.

        Args:
            session_id: Session ID
            messages: List of dicts with keys: topic, payload, qos, retained
        """
        try:
            await self._ensure_connected()

            publish_requests = []
            for msg in messages:
                pub_req = mqtt_service_pb2.PublishRequest(
                    user_id=self.user_id,
                    session_id=session_id,
                    topic=msg.get('topic'),
                    payload=msg.get('payload', b''),
                    qos=msg.get('qos', 1),
                    retained=msg.get('retained', False)
                )
                publish_requests.append(pub_req)

            request = mqtt_service_pb2.PublishBatchRequest(
                user_id=self.user_id,
                session_id=session_id,
                messages=publish_requests
            )

            response = await self.stub.PublishBatch(request)

            if response.success:
                return {
                    'success': True,
                    'published_count': response.published_count,
                    'failed_count': response.failed_count,
                    'message_ids': list(response.message_ids),
                    'errors': list(response.errors)
                }
            return None

        except Exception as e:
            return self.handle_error(e, "publish batch")

    async def publish_json(self, session_id: str, topic: str, data: Dict,
                          qos: int = 1, retained: bool = False) -> Optional[Dict]:
        """Publish JSON message."""
        try:
            await self._ensure_connected()

            struct_data = Struct()
            struct_data.update(data)

            request = mqtt_service_pb2.PublishJSONRequest(
                user_id=self.user_id,
                session_id=session_id,
                topic=topic,
                data=struct_data,
                qos=qos,
                retained=retained
            )

            response = await self.stub.PublishJSON(request)

            if response.success:
                return {'success': True, 'message_id': response.message_id}
            return None

        except Exception as e:
            return self.handle_error(e, "publish JSON")

    # ============================================
    # Subscriptions (Async Streaming)
    # ============================================

    async def subscribe(self, session_id: str, topic_filter: str,
                       qos: int = 1) -> AsyncIterator[Dict]:
        """
        Subscribe to topic (async streaming).

        Args:
            session_id: Session ID
            topic_filter: Topic filter (supports wildcards +, #)
            qos: QoS level

        Yields:
            Message dictionaries
        """
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.SubscribeRequest(
                user_id=self.user_id,
                session_id=session_id,
                topic_filter=topic_filter,
                qos=qos
            )

            async for message in self.stub.Subscribe(request):
                yield {
                    'topic': message.topic,
                    'payload': message.payload,
                    'qos': message.qos,
                    'retained': message.retained,
                    'timestamp': message.timestamp
                }

        except Exception as e:
            self.handle_error(e, "subscribe")

    async def subscribe_multiple(self, session_id: str,
                                subscriptions: List[Dict]) -> AsyncIterator[Dict]:
        """
        Subscribe to multiple topics (async streaming).

        Args:
            session_id: Session ID
            subscriptions: List of dicts with keys: topic_filter, qos

        Yields:
            Message dictionaries
        """
        try:
            await self._ensure_connected()

            topic_subs = []
            for sub in subscriptions:
                topic_sub = mqtt_service_pb2.TopicSubscription(
                    topic_filter=sub.get('topic_filter'),
                    qos=sub.get('qos', 1)
                )
                topic_subs.append(topic_sub)

            request = mqtt_service_pb2.SubscribeMultipleRequest(
                user_id=self.user_id,
                session_id=session_id,
                subscriptions=topic_subs
            )

            async for message in self.stub.SubscribeMultiple(request):
                yield {
                    'topic': message.topic,
                    'payload': message.payload,
                    'qos': message.qos,
                    'retained': message.retained,
                    'timestamp': message.timestamp
                }

        except Exception as e:
            self.handle_error(e, "subscribe multiple")

    async def unsubscribe(self, session_id: str,
                         topic_filters: List[str]) -> Optional[int]:
        """Unsubscribe from topics."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.UnsubscribeRequest(
                user_id=self.user_id,
                session_id=session_id,
                topic_filters=topic_filters
            )

            response = await self.stub.Unsubscribe(request)

            if response.success:
                return response.unsubscribed_count
            return None

        except Exception as e:
            return self.handle_error(e, "unsubscribe")

    async def list_subscriptions(self, session_id: str) -> List[Dict]:
        """List active subscriptions."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.ListSubscriptionsRequest(
                user_id=self.user_id,
                session_id=session_id
            )

            response = await self.stub.ListSubscriptions(request)

            subscriptions = []
            for sub in response.subscriptions:
                subscriptions.append({
                    'topic_filter': sub.topic_filter,
                    'qos': sub.qos
                })
            return subscriptions

        except Exception as e:
            return self.handle_error(e, "list subscriptions") or []

    # ============================================
    # Device Management
    # ============================================

    async def register_device(self, device_id: str, device_name: str,
                             device_type: str = 'sensor',
                             metadata: Optional[Dict[str, str]] = None) -> Optional[Dict]:
        """Register IoT device."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.RegisterDeviceRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                device_id=device_id,
                device_name=device_name,
                device_type=device_type,
                metadata=metadata or {}
            )

            response = await self.stub.RegisterDevice(request)

            if response.success:
                return {
                    'success': True,
                    'device': response.device,
                    'message': response.message
                }
            return None

        except Exception as e:
            return self.handle_error(e, "register device")

    async def unregister_device(self, device_id: str) -> bool:
        """Unregister device."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.UnregisterDeviceRequest(
                user_id=self.user_id,
                device_id=device_id
            )

            response = await self.stub.UnregisterDevice(request)
            return response.success

        except Exception as e:
            self.handle_error(e, "unregister device")
            return False

    async def list_devices(self, status: Optional[int] = None,
                          page: int = 1, page_size: int = 50) -> Optional[Dict]:
        """List registered devices."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.ListDevicesRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                status=status or 0,
                page=page,
                page_size=page_size
            )

            response = await self.stub.ListDevices(request)

            devices = []
            for device in response.devices:
                devices.append({
                    'device_id': device.device_id,
                    'device_name': device.device_name,
                    'device_type': device.device_type,
                    'status': device.status,
                    'registered_at': device.registered_at,
                    'last_seen': device.last_seen
                })

            return {
                'devices': devices,
                'total_count': response.total_count,
                'page': response.page,
                'page_size': response.page_size
            }

        except Exception as e:
            return self.handle_error(e, "list devices")

    async def get_device_info(self, device_id: str) -> Optional[Dict]:
        """Get device information."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.GetDeviceInfoRequest(
                user_id=self.user_id,
                device_id=device_id
            )

            response = await self.stub.GetDeviceInfo(request)

            device = response.device
            return {
                'device_id': device.device_id,
                'device_name': device.device_name,
                'device_type': device.device_type,
                'status': device.status,
                'registered_at': device.registered_at,
                'last_seen': device.last_seen,
                'metadata': dict(device.metadata),
                'subscribed_topics': list(device.subscribed_topics),
                'messages_sent': device.messages_sent,
                'messages_received': device.messages_received
            }

        except Exception as e:
            return self.handle_error(e, "get device info")

    async def update_device_status(self, device_id: str, status: int,
                                  metadata: Optional[Dict[str, str]] = None) -> Optional[Dict]:
        """Update device status."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.UpdateDeviceStatusRequest(
                user_id=self.user_id,
                device_id=device_id,
                status=status,
                metadata=metadata or {}
            )

            response = await self.stub.UpdateDeviceStatus(request)

            if response.success:
                return {'success': True, 'device': response.device}
            return None

        except Exception as e:
            return self.handle_error(e, "update device status")

    # ============================================
    # Topic Management
    # ============================================

    async def get_topic_info(self, topic: str) -> Optional[Dict]:
        """Get topic information."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.GetTopicInfoRequest(
                user_id=self.user_id,
                topic=topic
            )

            response = await self.stub.GetTopicInfo(request)

            topic_info = response.topic_info
            return {
                'topic': topic_info.topic,
                'subscriber_count': topic_info.subscriber_count,
                'message_count': topic_info.message_count,
                'last_message_time': topic_info.last_message_time,
                'has_retained_message': topic_info.has_retained_message
            }

        except Exception as e:
            return self.handle_error(e, "get topic info")

    async def list_topics(self, prefix: str = '', page: int = 1,
                         page_size: int = 50) -> Optional[Dict]:
        """List topics."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.ListTopicsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                prefix=prefix,
                page=page,
                page_size=page_size
            )

            response = await self.stub.ListTopics(request)

            topics = []
            for topic_info in response.topics:
                topics.append({
                    'topic': topic_info.topic,
                    'subscriber_count': topic_info.subscriber_count,
                    'message_count': topic_info.message_count
                })

            return {
                'topics': topics,
                'total_count': response.total_count
            }

        except Exception as e:
            return self.handle_error(e, "list topics")

    async def validate_topic(self, topic: str,
                            allow_wildcards: bool = False) -> Optional[Dict]:
        """Validate topic name."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.ValidateTopicRequest(
                topic=topic,
                allow_wildcards=allow_wildcards
            )

            response = await self.stub.ValidateTopic(request)

            return {
                'valid': response.valid,
                'message': response.message
            }

        except Exception as e:
            return self.handle_error(e, "validate topic")

    # ============================================
    # Retained Messages
    # ============================================

    async def set_retained_message(self, topic: str, payload: bytes,
                                  qos: int = 1) -> bool:
        """Set retained message."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.SetRetainedMessageRequest(
                user_id=self.user_id,
                topic=topic,
                payload=payload,
                qos=qos
            )

            response = await self.stub.SetRetainedMessage(request)
            return response.success

        except Exception as e:
            self.handle_error(e, "set retained message")
            return False

    async def get_retained_message(self, topic: str) -> Optional[Dict]:
        """Get retained message."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.GetRetainedMessageRequest(
                user_id=self.user_id,
                topic=topic
            )

            response = await self.stub.GetRetainedMessage(request)

            if response.found:
                message = response.message
                return {
                    'found': True,
                    'topic': message.topic,
                    'payload': message.payload,
                    'qos': message.qos,
                    'timestamp': message.timestamp
                }
            return {'found': False}

        except Exception as e:
            return self.handle_error(e, "get retained message")

    async def delete_retained_message(self, topic: str) -> bool:
        """Delete retained message."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.DeleteRetainedMessageRequest(
                user_id=self.user_id,
                topic=topic
            )

            response = await self.stub.DeleteRetainedMessage(request)
            return response.success

        except Exception as e:
            self.handle_error(e, "delete retained message")
            return False

    # ============================================
    # Statistics & Monitoring
    # ============================================

    async def get_statistics(self) -> Optional[Dict]:
        """Get statistics."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.GetStatisticsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id
            )

            response = await self.stub.GetStatistics(request)

            return {
                'total_devices': response.total_devices,
                'online_devices': response.online_devices,
                'total_topics': response.total_topics,
                'total_subscriptions': response.total_subscriptions,
                'messages_sent_today': response.messages_sent_today,
                'messages_received_today': response.messages_received_today,
                'active_sessions': response.active_sessions
            }

        except Exception as e:
            return self.handle_error(e, "get statistics")

    async def get_device_metrics(self, device_id: str,
                                start_time=None, end_time=None) -> Optional[Dict]:
        """Get device metrics."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.GetDeviceMetricsRequest(
                user_id=self.user_id,
                device_id=device_id,
                start_time=start_time,
                end_time=end_time
            )

            response = await self.stub.GetDeviceMetrics(request)

            return {
                'device_id': response.device_id,
                'messages_sent': response.messages_sent,
                'messages_received': response.messages_received,
                'bytes_sent': response.bytes_sent,
                'bytes_received': response.bytes_received,
                'message_rate': [(p.timestamp, p.value) for p in response.message_rate],
                'error_rate': [(p.timestamp, p.value) for p in response.error_rate]
            }

        except Exception as e:
            return self.handle_error(e, "get device metrics")

    # ============================================
    # Device Message Streaming
    # ============================================

    async def subscribe_device_messages(
        self,
        organization_id: Optional[str] = None,
        message_types: Optional[List[int]] = None,
        device_ids: Optional[List[str]] = None,
        topic_patterns: Optional[List[str]] = None
    ) -> AsyncIterator[Dict]:
        """
        Subscribe to all device messages (async streaming).

        Args:
            organization_id: Organization ID (optional, for filtering)
            message_types: Message types to subscribe (1=TELEMETRY, 2=STATUS, etc.)
            device_ids: Specific device IDs to subscribe
            topic_patterns: Custom topic patterns

        Yields:
            Device message dictionaries
        """
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.SubscribeDeviceMessagesRequest(
                user_id=self.user_id,
                organization_id=organization_id or '',
                message_types=message_types or [],
                device_ids=device_ids or [],
                topic_patterns=topic_patterns or []
            )

            async for message in self.stub.SubscribeDeviceMessages(request):
                yield {
                    'device_id': message.device_id,
                    'message_type': message.message_type,
                    'topic': message.topic,
                    'payload': message.payload,
                    'timestamp': message.timestamp,
                    'metadata': dict(message.metadata)
                }

        except Exception as e:
            self.handle_error(e, "subscribe device messages")

    # ============================================
    # Webhook Management
    # ============================================

    async def register_webhook(
        self,
        url: str,
        organization_id: Optional[str] = None,
        message_types: Optional[List[int]] = None,
        device_ids: Optional[List[str]] = None,
        topic_patterns: Optional[List[str]] = None,
        headers: Optional[Dict[str, str]] = None,
        secret: Optional[str] = None
    ) -> Optional[Dict]:
        """Register webhook for device messages."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.RegisterWebhookRequest(
                user_id=self.user_id,
                organization_id=organization_id or '',
                url=url,
                message_types=message_types or [],
                device_ids=device_ids or [],
                topic_patterns=topic_patterns or [],
                headers=headers or {},
                secret=secret or ''
            )

            response = await self.stub.RegisterWebhook(request)

            if response.success:
                return {
                    'success': True,
                    'webhook_id': response.webhook_id,
                    'webhook': {
                        'webhook_id': response.webhook.webhook_id,
                        'url': response.webhook.url,
                        'enabled': response.webhook.enabled,
                        'success_count': response.webhook.success_count,
                        'failure_count': response.webhook.failure_count,
                    }
                }
            return None

        except Exception as e:
            return self.handle_error(e, "register webhook")

    async def unregister_webhook(self, webhook_id: str) -> bool:
        """Unregister webhook."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.UnregisterWebhookRequest(
                user_id=self.user_id,
                webhook_id=webhook_id
            )

            response = await self.stub.UnregisterWebhook(request)
            return response.success

        except Exception as e:
            self.handle_error(e, "unregister webhook")
            return False

    async def list_webhooks(self, organization_id: Optional[str] = None,
                           include_disabled: bool = False) -> List[Dict]:
        """List registered webhooks."""
        try:
            await self._ensure_connected()
            request = mqtt_service_pb2.ListWebhooksRequest(
                user_id=self.user_id,
                organization_id=organization_id or '',
                include_disabled=include_disabled
            )

            response = await self.stub.ListWebhooks(request)

            webhooks = []
            for webhook in response.webhooks:
                webhooks.append({
                    'webhook_id': webhook.webhook_id,
                    'url': webhook.url,
                    'enabled': webhook.enabled,
                    'message_types': list(webhook.message_types),
                    'device_ids': list(webhook.device_ids),
                    'topic_patterns': list(webhook.topic_patterns),
                    'success_count': webhook.success_count,
                    'failure_count': webhook.failure_count,
                })
            return webhooks

        except Exception as e:
            return self.handle_error(e, "list webhooks") or []

    # ============================================
    # Concurrent Operations
    # ============================================

    async def publish_many_concurrent(self, session_id: str,
                                     messages: List[Dict]) -> List[Optional[Dict]]:
        """Publish multiple messages concurrently."""
        tasks = [
            self.publish(
                session_id,
                msg.get('topic'),
                msg.get('payload', b''),
                msg.get('qos', 1),
                msg.get('retained', False)
            )
            for msg in messages
        ]
        return await asyncio.gather(*tasks)

    async def register_devices_concurrent(self,
                                         devices: List[Dict]) -> List[Optional[Dict]]:
        """Register multiple devices concurrently."""
        tasks = [
            self.register_device(
                d.get('device_id'),
                d.get('device_name'),
                d.get('device_type', 'sensor'),
                d.get('metadata')
            )
            for d in devices
        ]
        return await asyncio.gather(*tasks)



#!/usr/bin/env python3
"""
MQTT 设备消息流测试
测试新增的 SubscribeDeviceMessages 和 Webhook 功能
"""

import time
import json
import threading
from isa_common import MQTTClient

# 测试配置
MQTT_HOST = 'localhost'
MQTT_PORT = 50053
USER_ID = 'test_user_001'
ORG_ID = 'test_org'

def test_device_message_stream():
    """
    测试 1: 订阅设备消息流（gRPC Streaming）
    """
    print("\n" + "="*60)
    print("测试 1: 设备消息流订阅 (gRPC Stream)")
    print("="*60)

    client = MQTTClient(
        host=MQTT_HOST,
        port=MQTT_PORT,
        user_id=USER_ID,
        organization_id=ORG_ID
    )

    # 消息计数器
    message_count = {'count': 0}

    def handle_device_message(device_id, message_type, topic, payload, timestamp, metadata):
        """处理接收到的设备消息"""
        message_count['count'] += 1
        print(f"\n📩 收到设备消息 #{message_count['count']}")
        print(f"   设备 ID: {device_id}")
        print(f"   消息类型: {message_type}")
        print(f"   Topic: {topic}")
        print(f"   Payload: {payload[:200]}")
        print(f"   时间戳: {timestamp}")
        print(f"   元数据: {metadata}")

    try:
        # 启动订阅（在后台线程运行）
        print("\n🔄 开始订阅设备消息...")
        print("   订阅消息类型: TELEMETRY (1), STATUS (2)")

        # 在单独线程中订阅
        subscribe_thread = threading.Thread(
            target=client.subscribe_device_messages,
            kwargs={
                'message_types': [1, 2],  # TELEMETRY and STATUS
                'callback': handle_device_message
            },
            daemon=True
        )
        subscribe_thread.start()

        # 等待一段时间接收消息
        print("\n⏳ 等待 10 秒接收设备消息...")
        print("   (请在另一个终端发送 MQTT 消息进行测试)")
        print("   示例: mosquitto_pub -t 'devices/test-device-001/telemetry' -m '{\"temp\": 25.5}'")

        time.sleep(10)

        print(f"\n✅ 测试完成！共接收 {message_count['count']} 条消息")

    except KeyboardInterrupt:
        print("\n\n⚠️  测试中断")
    except Exception as e:
        print(f"\n❌ 测试失败: {e}")
    finally:
        client.close()


def test_webhook_registration():
    """
    测试 2: Webhook 注册和管理
    """
    print("\n" + "="*60)
    print("测试 2: Webhook 注册和管理")
    print("="*60)

    client = MQTTClient(
        host=MQTT_HOST,
        port=MQTT_PORT,
        user_id=USER_ID,
        organization_id=ORG_ID
    )

    try:
        # 1. 注册 webhook
        print("\n📝 注册 Webhook...")
        webhook_result = client.register_webhook(
            url="http://localhost:8999/webhook/mqtt",  # 测试 URL
            message_types=[1, 2],  # TELEMETRY and STATUS
            topic_patterns=["devices/+/telemetry", "devices/+/status"],
            headers={"Authorization": "Bearer test-token"},
            secret="my-secret-key"
        )

        if webhook_result:
            webhook_id = webhook_result['webhook_id']
            print(f"✅ Webhook 注册成功!")
            print(f"   Webhook ID: {webhook_id}")
            print(f"   URL: {webhook_result['webhook']['url']}")
        else:
            print("❌ Webhook 注册失败")
            return

        # 2. 列出所有 webhooks
        print("\n📋 列出所有 Webhooks...")
        webhooks = client.list_webhooks(include_disabled=True)
        print(f"✅ 找到 {len(webhooks)} 个 Webhook:")
        for wh in webhooks:
            print(f"   - {wh['webhook_id']}: {wh['url']}")
            print(f"     成功: {wh['success_count']}, 失败: {wh['failure_count']}")

        # 3. 等待一段时间，让 webhook 接收消息
        print("\n⏳ 等待 10 秒测试 Webhook 回调...")
        print("   (请在另一个终端发送 MQTT 消息)")
        print("   或启动测试 webhook 服务器: python3 tests/test_webhook_server.py")
        time.sleep(10)

        # 4. 再次列出 webhooks 查看统计
        print("\n📊 查看 Webhook 统计...")
        webhooks = client.list_webhooks()
        for wh in webhooks:
            if wh['webhook_id'] == webhook_id:
                print(f"✅ Webhook 统计:")
                print(f"   成功回调: {wh['success_count']}")
                print(f"   失败回调: {wh['failure_count']}")

        # 5. 注销 webhook
        print(f"\n🗑️  注销 Webhook {webhook_id}...")
        if client.unregister_webhook(webhook_id):
            print("✅ Webhook 注销成功")
        else:
            print("❌ Webhook 注销失败")

    except Exception as e:
        print(f"\n❌ 测试失败: {e}")
        import traceback
        traceback.print_exc()
    finally:
        client.close()


def test_simulated_device():
    """
    测试 3: 模拟设备发送消息
    """
    print("\n" + "="*60)
    print("测试 3: 模拟设备发送消息")
    print("="*60)

    client = MQTTClient(
        host=MQTT_HOST,
        port=MQTT_PORT,
        user_id=USER_ID,
        organization_id=ORG_ID
    )

    try:
        # 连接
        print("\n🔌 连接到 MQTT 服务...")
        conn = client.connect('test-device-simulator')

        if not conn:
            print("❌ 连接失败")
            return

        session_id = conn['session_id']
        print(f"✅ 连接成功! Session ID: {session_id}")

        # 发送遥测数据
        print("\n📤 发送设备遥测数据...")
        telemetry_data = {
            "device_id": "test-device-001",
            "temperature": 25.5,
            "humidity": 60.2,
            "timestamp": int(time.time())
        }

        client.publish_json(
            session_id=session_id,
            topic="devices/test-device-001/telemetry",
            data=telemetry_data,
            qos=1
        )
        print("✅ 遥测数据已发送")

        # 发送状态更新
        print("\n📤 发送设备状态...")
        status_data = {
            "device_id": "test-device-001",
            "status": "online",
            "battery": 85,
            "timestamp": int(time.time())
        }

        client.publish_json(
            session_id=session_id,
            topic="devices/test-device-001/status",
            data=status_data,
            qos=1
        )
        print("✅ 状态数据已发送")

        # 等待一下让消息被处理
        time.sleep(2)

        # 断开连接
        print("\n🔌 断开连接...")
        client.disconnect(session_id)
        print("✅ 断开成功")

    except Exception as e:
        print(f"\n❌ 测试失败: {e}")
        import traceback
        traceback.print_exc()
    finally:
        client.close()


if __name__ == '__main__':
    print("\n" + "="*60)
    print("MQTT 设备消息流端到端测试")
    print("="*60)
    print("\n测试目标:")
    print("  1. gRPC Stream - 订阅设备消息")
    print("  2. Webhook - 注册和管理")
    print("  3. 模拟设备 - 发送消息")
    print("\n前置条件:")
    print("  - MQTT Broker 运行在 localhost:1883")
    print("  - mqtt-service 运行在 localhost:50053")
    print("")

    # 运行测试
    try:
        # 测试 3: 先发送一些消息
        test_simulated_device()

        # 测试 2: Webhook
        test_webhook_registration()

        # 测试 1: Stream (会一直监听，需要手动中断)
        test_device_message_stream()

    except KeyboardInterrupt:
        print("\n\n✅ 测试完成!")



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from . import nats_service_pb2 as nats__service__pb2

GRPC_GENERATED_VERSION = '1.75.1'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in nats_service_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class NATSServiceStub(object):
    """========================================
    NATS 消息流服务
    ========================================
    这是一个统一的消息流服务，为所有后端服务提供事件总线能力
    支持：
    - 用户和组织级别的主题隔离
    - Pub/Sub 消息
    - 请求/响应模式
    - JetStream 持久化
    - 流管理
    - 消费者管理
    - 队列组

    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Publish = channel.unary_unary(
                '/isa.nats.NATSService/Publish',
                request_serializer=nats__service__pb2.PublishRequest.SerializeToString,
                response_deserializer=nats__service__pb2.PublishResponse.FromString,
                _registered_method=True)
        self.PublishBatch = channel.unary_unary(
                '/isa.nats.NATSService/PublishBatch',
                request_serializer=nats__service__pb2.PublishBatchRequest.SerializeToString,
                response_deserializer=nats__service__pb2.PublishBatchResponse.FromString,
                _registered_method=True)
        self.Subscribe = channel.unary_stream(
                '/isa.nats.NATSService/Subscribe',
                request_serializer=nats__service__pb2.SubscribeRequest.SerializeToString,
                response_deserializer=nats__service__pb2.MessageResponse.FromString,
                _registered_method=True)
        self.Unsubscribe = channel.unary_unary(
                '/isa.nats.NATSService/Unsubscribe',
                request_serializer=nats__service__pb2.UnsubscribeRequest.SerializeToString,
                response_deserializer=nats__service__pb2.UnsubscribeResponse.FromString,
                _registered_method=True)
        self.Request = channel.unary_unary(
                '/isa.nats.NATSService/Request',
                request_serializer=nats__service__pb2.RequestRequest.SerializeToString,
                response_deserializer=nats__service__pb2.RequestResponse.FromString,
                _registered_method=True)
        self.QueueSubscribe = channel.unary_stream(
                '/isa.nats.NATSService/QueueSubscribe',
                request_serializer=nats__service__pb2.QueueSubscribeRequest.SerializeToString,
                response_deserializer=nats__service__pb2.MessageResponse.FromString,
                _registered_method=True)
        self.CreateStream = channel.unary_unary(
                '/isa.nats.NATSService/CreateStream',
                request_serializer=nats__service__pb2.CreateStreamRequest.SerializeToString,
                response_deserializer=nats__service__pb2.CreateStreamResponse.FromString,
                _registered_method=True)
        self.DeleteStream = channel.unary_unary(
                '/isa.nats.NATSService/DeleteStream',
                request_serializer=nats__service__pb2.DeleteStreamRequest.SerializeToString,
                response_deserializer=nats__service__pb2.DeleteStreamResponse.FromString,
                _registered_method=True)
        self.GetStreamInfo = channel.unary_unary(
                '/isa.nats.NATSService/GetStreamInfo',
                request_serializer=nats__service__pb2.GetStreamInfoRequest.SerializeToString,
                response_deserializer=nats__service__pb2.GetStreamInfoResponse.FromString,
                _registered_method=True)
        self.ListStreams = channel.unary_unary(
                '/isa.nats.NATSService/ListStreams',
                request_serializer=nats__service__pb2.ListStreamsRequest.SerializeToString,
                response_deserializer=nats__service__pb2.ListStreamsResponse.FromString,
                _registered_method=True)
        self.UpdateStream = channel.unary_unary(
                '/isa.nats.NATSService/UpdateStream',
                request_serializer=nats__service__pb2.UpdateStreamRequest.SerializeToString,
                response_deserializer=nats__service__pb2.UpdateStreamResponse.FromString,
                _registered_method=True)
        self.PurgeStream = channel.unary_unary(
                '/isa.nats.NATSService/PurgeStream',
                request_serializer=nats__service__pb2.PurgeStreamRequest.SerializeToString,
                response_deserializer=nats__service__pb2.PurgeStreamResponse.FromString,
                _registered_method=True)
        self.CreateConsumer = channel.unary_unary(
                '/isa.nats.NATSService/CreateConsumer',
                request_serializer=nats__service__pb2.CreateConsumerRequest.SerializeToString,
                response_deserializer=nats__service__pb2.CreateConsumerResponse.FromString,
                _registered_method=True)
        self.DeleteConsumer = channel.unary_unary(
                '/isa.nats.NATSService/DeleteConsumer',
                request_serializer=nats__service__pb2.DeleteConsumerRequest.SerializeToString,
                response_deserializer=nats__service__pb2.DeleteConsumerResponse.FromString,
                _registered_method=True)
        self.GetConsumerInfo = channel.unary_unary(
                '/isa.nats.NATSService/GetConsumerInfo',
                request_serializer=nats__service__pb2.GetConsumerInfoRequest.SerializeToString,
                response_deserializer=nats__service__pb2.GetConsumerInfoResponse.FromString,
                _registered_method=True)
        self.ListConsumers = channel.unary_unary(
                '/isa.nats.NATSService/ListConsumers',
                request_serializer=nats__service__pb2.ListConsumersRequest.SerializeToString,
                response_deserializer=nats__service__pb2.ListConsumersResponse.FromString,
                _registered_method=True)
        self.PublishToStream = channel.unary_unary(
                '/isa.nats.NATSService/PublishToStream',
                request_serializer=nats__service__pb2.PublishToStreamRequest.SerializeToString,
                response_deserializer=nats__service__pb2.PublishToStreamResponse.FromString,
                _registered_method=True)
        self.PullMessages = channel.unary_unary(
                '/isa.nats.NATSService/PullMessages',
                request_serializer=nats__service__pb2.PullMessagesRequest.SerializeToString,
                response_deserializer=nats__service__pb2.PullMessagesResponse.FromString,
                _registered_method=True)
        self.AckMessage = channel.unary_unary(
                '/isa.nats.NATSService/AckMessage',
                request_serializer=nats__service__pb2.AckMessageRequest.SerializeToString,
                response_deserializer=nats__service__pb2.AckMessageResponse.FromString,
                _registered_method=True)
        self.NakMessage = channel.unary_unary(
                '/isa.nats.NATSService/NakMessage',
                request_serializer=nats__service__pb2.NakMessageRequest.SerializeToString,
                response_deserializer=nats__service__pb2.NakMessageResponse.FromString,
                _registered_method=True)
        self.KVPut = channel.unary_unary(
                '/isa.nats.NATSService/KVPut',
                request_serializer=nats__service__pb2.KVPutRequest.SerializeToString,
                response_deserializer=nats__service__pb2.KVPutResponse.FromString,
                _registered_method=True)
        self.KVGet = channel.unary_unary(
                '/isa.nats.NATSService/KVGet',
                request_serializer=nats__service__pb2.KVGetRequest.SerializeToString,
                response_deserializer=nats__service__pb2.KVGetResponse.FromString,
                _registered_method=True)
        self.KVDelete = channel.unary_unary(
                '/isa.nats.NATSService/KVDelete',
                request_serializer=nats__service__pb2.KVDeleteRequest.SerializeToString,
                response_deserializer=nats__service__pb2.KVDeleteResponse.FromString,
                _registered_method=True)
        self.KVKeys = channel.unary_unary(
                '/isa.nats.NATSService/KVKeys',
                request_serializer=nats__service__pb2.KVKeysRequest.SerializeToString,
                response_deserializer=nats__service__pb2.KVKeysResponse.FromString,
                _registered_method=True)
        self.ObjectPut = channel.unary_unary(
                '/isa.nats.NATSService/ObjectPut',
                request_serializer=nats__service__pb2.ObjectPutRequest.SerializeToString,
                response_deserializer=nats__service__pb2.ObjectPutResponse.FromString,
                _registered_method=True)
        self.ObjectGet = channel.unary_unary(
                '/isa.nats.NATSService/ObjectGet',
                request_serializer=nats__service__pb2.ObjectGetRequest.SerializeToString,
                response_deserializer=nats__service__pb2.ObjectGetResponse.FromString,
                _registered_method=True)
        self.ObjectDelete = channel.unary_unary(
                '/isa.nats.NATSService/ObjectDelete',
                request_serializer=nats__service__pb2.ObjectDeleteRequest.SerializeToString,
                response_deserializer=nats__service__pb2.ObjectDeleteResponse.FromString,
                _registered_method=True)
        self.ObjectList = channel.unary_unary(
                '/isa.nats.NATSService/ObjectList',
                request_serializer=nats__service__pb2.ObjectListRequest.SerializeToString,
                response_deserializer=nats__service__pb2.ObjectListResponse.FromString,
                _registered_method=True)
        self.GetStatistics = channel.unary_unary(
                '/isa.nats.NATSService/GetStatistics',
                request_serializer=nats__service__pb2.GetStatisticsRequest.SerializeToString,
                response_deserializer=nats__service__pb2.GetStatisticsResponse.FromString,
                _registered_method=True)
        self.GetStreamStats = channel.unary_unary(
                '/isa.nats.NATSService/GetStreamStats',
                request_serializer=nats__service__pb2.GetStreamStatsRequest.SerializeToString,
                response_deserializer=nats__service__pb2.GetStreamStatsResponse.FromString,
                _registered_method=True)
        self.HealthCheck = channel.unary_unary(
                '/isa.nats.NATSService/HealthCheck',
                request_serializer=nats__service__pb2.NATSHealthCheckRequest.SerializeToString,
                response_deserializer=nats__service__pb2.NATSHealthCheckResponse.FromString,
                _registered_method=True)


class NATSServiceServicer(object):
    """========================================
    NATS 消息流服务
    ========================================
    这是一个统一的消息流服务，为所有后端服务提供事件总线能力
    支持：
    - 用户和组织级别的主题隔离
    - Pub/Sub 消息
    - 请求/响应模式
    - JetStream 持久化
    - 流管理
    - 消费者管理
    - 队列组

    """

    def Publish(self, request, context):
        """========== 基础 Pub/Sub ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PublishBatch(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Subscribe(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Unsubscribe(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Request(self, request, context):
        """========== 请求/响应模式 ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueueSubscribe(self, request, context):
        """========== 队列组订阅 ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateStream(self, request, context):
        """========== JetStream 流管理 ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteStream(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetStreamInfo(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListStreams(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateStream(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PurgeStream(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateConsumer(self, request, context):
        """========== JetStream 消费者管理 ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteConsumer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetConsumerInfo(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListConsumers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PublishToStream(self, request, context):
        """========== JetStream 消息操作 ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PullMessages(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AckMessage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def NakMessage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def KVPut(self, request, context):
        """========== 键值存储 (KV Store) ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def KVGet(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def KVDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def KVKeys(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ObjectPut(self, request, context):
        """========== 对象存储 (Object Store) ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ObjectGet(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ObjectDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ObjectList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetStatistics(self, request, context):
        """========== 统计和监控 ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetStreamStats(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def HealthCheck(self, request, context):
        """========== 健康检查 ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_NATSServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Publish': grpc.unary_unary_rpc_method_handler(
                    servicer.Publish,
                    request_deserializer=nats__service__pb2.PublishRequest.FromString,
                    response_serializer=nats__service__pb2.PublishResponse.SerializeToString,
            ),
            'PublishBatch': grpc.unary_unary_rpc_method_handler(
                    servicer.PublishBatch,
                    request_deserializer=nats__service__pb2.PublishBatchRequest.FromString,
                    response_serializer=nats__service__pb2.PublishBatchResponse.SerializeToString,
            ),
            'Subscribe': grpc.unary_stream_rpc_method_handler(
                    servicer.Subscribe,
                    request_deserializer=nats__service__pb2.SubscribeRequest.FromString,
                    response_serializer=nats__service__pb2.MessageResponse.SerializeToString,
            ),
            'Unsubscribe': grpc.unary_unary_rpc_method_handler(
                    servicer.Unsubscribe,
                    request_deserializer=nats__service__pb2.UnsubscribeRequest.FromString,
                    response_serializer=nats__service__pb2.UnsubscribeResponse.SerializeToString,
            ),
            'Request': grpc.unary_unary_rpc_method_handler(
                    servicer.Request,
                    request_deserializer=nats__service__pb2.RequestRequest.FromString,
                    response_serializer=nats__service__pb2.RequestResponse.SerializeToString,
            ),
            'QueueSubscribe': grpc.unary_stream_rpc_method_handler(
                    servicer.QueueSubscribe,
                    request_deserializer=nats__service__pb2.QueueSubscribeRequest.FromString,
                    response_serializer=nats__service__pb2.MessageResponse.SerializeToString,
            ),
            'CreateStream': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateStream,
                    request_deserializer=nats__service__pb2.CreateStreamRequest.FromString,
                    response_serializer=nats__service__pb2.CreateStreamResponse.SerializeToString,
            ),
            'DeleteStream': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteStream,
                    request_deserializer=nats__service__pb2.DeleteStreamRequest.FromString,
                    response_serializer=nats__service__pb2.DeleteStreamResponse.SerializeToString,
            ),
            'GetStreamInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStreamInfo,
                    request_deserializer=nats__service__pb2.GetStreamInfoRequest.FromString,
                    response_serializer=nats__service__pb2.GetStreamInfoResponse.SerializeToString,
            ),
            'ListStreams': grpc.unary_unary_rpc_method_handler(
                    servicer.ListStreams,
                    request_deserializer=nats__service__pb2.ListStreamsRequest.FromString,
                    response_serializer=nats__service__pb2.ListStreamsResponse.SerializeToString,
            ),
            'UpdateStream': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateStream,
                    request_deserializer=nats__service__pb2.UpdateStreamRequest.FromString,
                    response_serializer=nats__service__pb2.UpdateStreamResponse.SerializeToString,
            ),
            'PurgeStream': grpc.unary_unary_rpc_method_handler(
                    servicer.PurgeStream,
                    request_deserializer=nats__service__pb2.PurgeStreamRequest.FromString,
                    response_serializer=nats__service__pb2.PurgeStreamResponse.SerializeToString,
            ),
            'CreateConsumer': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateConsumer,
                    request_deserializer=nats__service__pb2.CreateConsumerRequest.FromString,
                    response_serializer=nats__service__pb2.CreateConsumerResponse.SerializeToString,
            ),
            'DeleteConsumer': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteConsumer,
                    request_deserializer=nats__service__pb2.DeleteConsumerRequest.FromString,
                    response_serializer=nats__service__pb2.DeleteConsumerResponse.SerializeToString,
            ),
            'GetConsumerInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetConsumerInfo,
                    request_deserializer=nats__service__pb2.GetConsumerInfoRequest.FromString,
                    response_serializer=nats__service__pb2.GetConsumerInfoResponse.SerializeToString,
            ),
            'ListConsumers': grpc.unary_unary_rpc_method_handler(
                    servicer.ListConsumers,
                    request_deserializer=nats__service__pb2.ListConsumersRequest.FromString,
                    response_serializer=nats__service__pb2.ListConsumersResponse.SerializeToString,
            ),
            'PublishToStream': grpc.unary_unary_rpc_method_handler(
                    servicer.PublishToStream,
                    request_deserializer=nats__service__pb2.PublishToStreamRequest.FromString,
                    response_serializer=nats__service__pb2.PublishToStreamResponse.SerializeToString,
            ),
            'PullMessages': grpc.unary_unary_rpc_method_handler(
                    servicer.PullMessages,
                    request_deserializer=nats__service__pb2.PullMessagesRequest.FromString,
                    response_serializer=nats__service__pb2.PullMessagesResponse.SerializeToString,
            ),
            'AckMessage': grpc.unary_unary_rpc_method_handler(
                    servicer.AckMessage,
                    request_deserializer=nats__service__pb2.AckMessageRequest.FromString,
                    response_serializer=nats__service__pb2.AckMessageResponse.SerializeToString,
            ),
            'NakMessage': grpc.unary_unary_rpc_method_handler(
                    servicer.NakMessage,
                    request_deserializer=nats__service__pb2.NakMessageRequest.FromString,
                    response_serializer=nats__service__pb2.NakMessageResponse.SerializeToString,
            ),
            'KVPut': grpc.unary_unary_rpc_method_handler(
                    servicer.KVPut,
                    request_deserializer=nats__service__pb2.KVPutRequest.FromString,
                    response_serializer=nats__service__pb2.KVPutResponse.SerializeToString,
            ),
            'KVGet': grpc.unary_unary_rpc_method_handler(
                    servicer.KVGet,
                    request_deserializer=nats__service__pb2.KVGetRequest.FromString,
                    response_serializer=nats__service__pb2.KVGetResponse.SerializeToString,
            ),
            'KVDelete': grpc.unary_unary_rpc_method_handler(
                    servicer.KVDelete,
                    request_deserializer=nats__service__pb2.KVDeleteRequest.FromString,
                    response_serializer=nats__service__pb2.KVDeleteResponse.SerializeToString,
            ),
            'KVKeys': grpc.unary_unary_rpc_method_handler(
                    servicer.KVKeys,
                    request_deserializer=nats__service__pb2.KVKeysRequest.FromString,
                    response_serializer=nats__service__pb2.KVKeysResponse.SerializeToString,
            ),
            'ObjectPut': grpc.unary_unary_rpc_method_handler(
                    servicer.ObjectPut,
                    request_deserializer=nats__service__pb2.ObjectPutRequest.FromString,
                    response_serializer=nats__service__pb2.ObjectPutResponse.SerializeToString,
            ),
            'ObjectGet': grpc.unary_unary_rpc_method_handler(
                    servicer.ObjectGet,
                    request_deserializer=nats__service__pb2.ObjectGetRequest.FromString,
                    response_serializer=nats__service__pb2.ObjectGetResponse.SerializeToString,
            ),
            'ObjectDelete': grpc.unary_unary_rpc_method_handler(
                    servicer.ObjectDelete,
                    request_deserializer=nats__service__pb2.ObjectDeleteRequest.FromString,
                    response_serializer=nats__service__pb2.ObjectDeleteResponse.SerializeToString,
            ),
            'ObjectList': grpc.unary_unary_rpc_method_handler(
                    servicer.ObjectList,
                    request_deserializer=nats__service__pb2.ObjectListRequest.FromString,
                    response_serializer=nats__service__pb2.ObjectListResponse.SerializeToString,
            ),
            'GetStatistics': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStatistics,
                    request_deserializer=nats__service__pb2.GetStatisticsRequest.FromString,
                    response_serializer=nats__service__pb2.GetStatisticsResponse.SerializeToString,
            ),
            'GetStreamStats': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStreamStats,
                    request_deserializer=nats__service__pb2.GetStreamStatsRequest.FromString,
                    response_serializer=nats__service__pb2.GetStreamStatsResponse.SerializeToString,
            ),
            'HealthCheck': grpc.unary_unary_rpc_method_handler(
                    servicer.HealthCheck,
                    request_deserializer=nats__service__pb2.NATSHealthCheckRequest.FromString,
                    response_serializer=nats__service__pb2.NATSHealthCheckResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'isa.nats.NATSService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('isa.nats.NATSService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class NATSService(object):
    """========================================
    NATS 消息流服务
    ========================================
    这是一个统一的消息流服务，为所有后端服务提供事件总线能力
    支持：
    - 用户和组织级别的主题隔离
    - Pub/Sub 消息
    - 请求/响应模式
    - JetStream 持久化
    - 流管理
    - 消费者管理
    - 队列组

    """

    @staticmethod
    def Publish(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/Publish',
            nats__service__pb2.PublishRequest.SerializeToString,
            nats__service__pb2.PublishResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def PublishBatch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/PublishBatch',
            nats__service__pb2.PublishBatchRequest.SerializeToString,
            nats__service__pb2.PublishBatchResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Subscribe(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/isa.nats.NATSService/Subscribe',
            nats__service__pb2.SubscribeRequest.SerializeToString,
            nats__service__pb2.MessageResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Unsubscribe(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/Unsubscribe',
            nats__service__pb2.UnsubscribeRequest.SerializeToString,
            nats__service__pb2.UnsubscribeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Request(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/Request',
            nats__service__pb2.RequestRequest.SerializeToString,
            nats__service__pb2.RequestResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def QueueSubscribe(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/isa.nats.NATSService/QueueSubscribe',
            nats__service__pb2.QueueSubscribeRequest.SerializeToString,
            nats__service__pb2.MessageResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CreateStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/CreateStream',
            nats__service__pb2.CreateStreamRequest.SerializeToString,
            nats__service__pb2.CreateStreamResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/DeleteStream',
            nats__service__pb2.DeleteStreamRequest.SerializeToString,
            nats__service__pb2.DeleteStreamResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetStreamInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/GetStreamInfo',
            nats__service__pb2.GetStreamInfoRequest.SerializeToString,
            nats__service__pb2.GetStreamInfoResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListStreams(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/ListStreams',
            nats__service__pb2.ListStreamsRequest.SerializeToString,
            nats__service__pb2.ListStreamsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def UpdateStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/UpdateStream',
            nats__service__pb2.UpdateStreamRequest.SerializeToString,
            nats__service__pb2.UpdateStreamResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def PurgeStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/PurgeStream',
            nats__service__pb2.PurgeStreamRequest.SerializeToString,
            nats__service__pb2.PurgeStreamResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CreateConsumer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/CreateConsumer',
            nats__service__pb2.CreateConsumerRequest.SerializeToString,
            nats__service__pb2.CreateConsumerResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteConsumer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/DeleteConsumer',
            nats__service__pb2.DeleteConsumerRequest.SerializeToString,
            nats__service__pb2.DeleteConsumerResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetConsumerInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/GetConsumerInfo',
            nats__service__pb2.GetConsumerInfoRequest.SerializeToString,
            nats__service__pb2.GetConsumerInfoResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListConsumers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/ListConsumers',
            nats__service__pb2.ListConsumersRequest.SerializeToString,
            nats__service__pb2.ListConsumersResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def PublishToStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/PublishToStream',
            nats__service__pb2.PublishToStreamRequest.SerializeToString,
            nats__service__pb2.PublishToStreamResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def PullMessages(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/PullMessages',
            nats__service__pb2.PullMessagesRequest.SerializeToString,
            nats__service__pb2.PullMessagesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def AckMessage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/AckMessage',
            nats__service__pb2.AckMessageRequest.SerializeToString,
            nats__service__pb2.AckMessageResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def NakMessage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/NakMessage',
            nats__service__pb2.NakMessageRequest.SerializeToString,
            nats__service__pb2.NakMessageResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def KVPut(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/KVPut',
            nats__service__pb2.KVPutRequest.SerializeToString,
            nats__service__pb2.KVPutResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def KVGet(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/KVGet',
            nats__service__pb2.KVGetRequest.SerializeToString,
            nats__service__pb2.KVGetResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def KVDelete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/KVDelete',
            nats__service__pb2.KVDeleteRequest.SerializeToString,
            nats__service__pb2.KVDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def KVKeys(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/KVKeys',
            nats__service__pb2.KVKeysRequest.SerializeToString,
            nats__service__pb2.KVKeysResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ObjectPut(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/ObjectPut',
            nats__service__pb2.ObjectPutRequest.SerializeToString,
            nats__service__pb2.ObjectPutResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ObjectGet(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/ObjectGet',
            nats__service__pb2.ObjectGetRequest.SerializeToString,
            nats__service__pb2.ObjectGetResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ObjectDelete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/ObjectDelete',
            nats__service__pb2.ObjectDeleteRequest.SerializeToString,
            nats__service__pb2.ObjectDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ObjectList(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/ObjectList',
            nats__service__pb2.ObjectListRequest.SerializeToString,
            nats__service__pb2.ObjectListResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetStatistics(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/GetStatistics',
            nats__service__pb2.GetStatisticsRequest.SerializeToString,
            nats__service__pb2.GetStatisticsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetStreamStats(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/GetStreamStats',
            nats__service__pb2.GetStreamStatsRequest.SerializeToString,
            nats__service__pb2.GetStreamStatsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def HealthCheck(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.nats.NATSService/HealthCheck',
            nats__service__pb2.NATSHealthCheckRequest.SerializeToString,
            nats__service__pb2.NATSHealthCheckResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



#!/usr/bin/env python3
"""
Async NATS gRPC Client
High-performance async NATS client using grpc.aio

Performance Benefits:
- True async I/O without GIL blocking
- Concurrent message publishing
- Efficient streaming subscriptions
- Memory-efficient connection pooling
"""

import logging
from typing import List, Dict, Optional, AsyncIterator, Callable, TYPE_CHECKING
from .async_base_client import AsyncBaseGRPCClient
from .proto import nats_service_pb2, nats_service_pb2_grpc
from google.protobuf.duration_pb2 import Duration

if TYPE_CHECKING:
    from .consul_client import ConsulRegistry

logger = logging.getLogger(__name__)


class AsyncNATSClient(AsyncBaseGRPCClient):
    """Async NATS gRPC client for high-performance messaging."""

    def __init__(
        self,
        host: Optional[str] = None,
        port: Optional[int] = None,
        user_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        lazy_connect: bool = True,
        enable_compression: bool = False,
        enable_retry: bool = True,
        consul_registry: Optional['ConsulRegistry'] = None,
        service_name_override: Optional[str] = None
    ):
        """
        Initialize async NATS client.

        Args:
            host: Service address (optional, will use Consul discovery if not provided)
            port: Service port (optional, will use Consul discovery if not provided)
            user_id: User ID
            organization_id: Organization ID
            lazy_connect: Lazy connection (default: True)
            enable_compression: Enable compression (default: False for low latency)
            enable_retry: Enable retry (default: True)
            consul_registry: ConsulRegistry instance for service discovery (optional)
            service_name_override: Override service name for Consul lookup (optional)
        """
        super().__init__(
            host=host,
            port=port,
            user_id=user_id,
            lazy_connect=lazy_connect,
            enable_compression=enable_compression,
            enable_retry=enable_retry,
            consul_registry=consul_registry,
            service_name_override=service_name_override
        )
        self.organization_id = organization_id or 'default-org'

    def _create_stub(self):
        """Create NATS service stub."""
        return nats_service_pb2_grpc.NATSServiceStub(self.channel)

    def service_name(self) -> str:
        return "NATS"

    def default_port(self) -> int:
        return 50056

    # ============================================
    # Health Check
    # ============================================

    async def health_check(self, deep_check: bool = False) -> Optional[Dict]:
        """Health check."""
        try:
            await self._ensure_connected()
            request = nats_service_pb2.NATSHealthCheckRequest(deep_check=deep_check)
            response = await self.stub.HealthCheck(request)

            return {
                'healthy': response.healthy,
                'nats_status': response.nats_status,
                'jetstream_enabled': response.jetstream_enabled,
                'connections': response.connections,
                'message': response.message
            }

        except Exception as e:
            return self.handle_error(e, "health check")

    # ============================================
    # Core Publish/Subscribe
    # ============================================

    async def publish(
        self,
        subject: str,
        data: bytes,
        headers: Optional[Dict[str, str]] = None,
        reply_to: str = ''
    ) -> Optional[Dict]:
        """
        Publish message to subject (fire-and-forget, ephemeral).

        Args:
            subject: Subject to publish to (lowercase.dots)
            data: Message payload
            headers: Optional headers
            reply_to: Optional reply-to subject

        Returns:
            {'success': True, 'message': str} or None
        """
        try:
            await self._ensure_connected()

            request = nats_service_pb2.PublishRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                subject=subject,
                data=data,
                headers=headers or {},
                reply_to=reply_to
            )

            response = await self.stub.Publish(request)

            if response.success:
                return {'success': True, 'message': response.message}
            return None

        except Exception as e:
            return self.handle_error(e, "publish")

    async def publish_batch(self, messages: List[Dict]) -> Optional[Dict]:
        """
        Batch publish multiple messages.

        Args:
            messages: List of dicts with 'subject' and 'data' keys

        Returns:
            {'success': True, 'published_count': int, 'errors': list}
        """
        try:
            await self._ensure_connected()

            nats_messages = []
            for msg in messages:
                nats_msg = nats_service_pb2.NATSMessage(
                    subject=msg['subject'],
                    data=msg['data'],
                    headers=msg.get('headers', {}),
                    reply_to=msg.get('reply_to', '')
                )
                nats_messages.append(nats_msg)

            request = nats_service_pb2.PublishBatchRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                messages=nats_messages
            )

            response = await self.stub.PublishBatch(request)

            if response.success:
                return {
                    'success': True,
                    'published_count': response.published_count,
                    'errors': list(response.errors)
                }
            return None

        except Exception as e:
            return self.handle_error(e, "publish batch")

    async def subscribe(self, subject: str, queue_group: str = '') -> AsyncIterator[Dict]:
        """
        Subscribe to a subject and yield messages.

        Args:
            subject: Subject to subscribe to (supports wildcards: *, >)
            queue_group: Optional queue group for load balancing

        Yields:
            Message dicts with 'subject', 'data', 'headers', 'reply_to', 'sequence'
        """
        try:
            await self._ensure_connected()

            request = nats_service_pb2.SubscribeRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                subject=subject,
                queue_group=queue_group
            )

            async for message_response in self.stub.Subscribe(request):
                yield {
                    'subject': message_response.subject,
                    'data': message_response.data,
                    'headers': dict(message_response.headers),
                    'reply_to': message_response.reply_to,
                    'sequence': message_response.sequence
                }

        except Exception as e:
            self.handle_error(e, "subscribe")

    async def request(self, subject: str, data: bytes, timeout_seconds: int = 5) -> Optional[Dict]:
        """Request-reply pattern."""
        try:
            await self._ensure_connected()
            timeout = Duration(seconds=timeout_seconds)

            request = nats_service_pb2.RequestRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                subject=subject,
                data=data,
                timeout=timeout
            )

            response = await self.stub.Request(request)

            if response.success:
                return {
                    'success': True,
                    'data': response.data,
                    'subject': response.subject
                }
            return None

        except Exception as e:
            return self.handle_error(e, "request")

    # ============================================
    # JetStream Operations
    # ============================================

    async def create_stream(
        self,
        name: str,
        subjects: List[str],
        max_msgs: int = -1,
        max_bytes: int = -1
    ) -> Optional[Dict]:
        """
        Create JetStream stream.

        Args:
            name: Stream name (should be UPPERCASE_UNDERSCORE)
            subjects: List of subjects to capture (lowercase.dots)
            max_msgs: Maximum number of messages (-1 for unlimited)
            max_bytes: Maximum bytes (-1 for unlimited)

        Returns:
            Stream info dict or None
        """
        try:
            await self._ensure_connected()

            config = nats_service_pb2.StreamConfig(
                name=name,
                subjects=subjects,
                storage=nats_service_pb2.STORAGE_FILE,
                max_msgs=max_msgs,
                max_bytes=max_bytes
            )

            request = nats_service_pb2.CreateStreamRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                config=config
            )

            response = await self.stub.CreateStream(request)

            if response.success:
                return {
                    'success': True,
                    'stream': {
                        'name': response.stream.name,
                        'subjects': list(response.stream.config.subjects),
                        'messages': response.stream.state.messages
                    }
                }
            return None

        except Exception as e:
            return self.handle_error(e, "create stream")

    async def delete_stream(self, stream_name: str) -> Optional[Dict]:
        """Delete JetStream stream."""
        try:
            await self._ensure_connected()
            request = nats_service_pb2.DeleteStreamRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                stream_name=stream_name
            )

            response = await self.stub.DeleteStream(request)

            if response.success:
                return {'success': True}
            return None

        except Exception as e:
            return self.handle_error(e, "delete stream")

    async def list_streams(self) -> List[Dict]:
        """List all streams."""
        try:
            await self._ensure_connected()
            request = nats_service_pb2.ListStreamsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id
            )

            response = await self.stub.ListStreams(request)

            return [
                {
                    'name': stream.name,
                    'subjects': list(stream.subjects),
                    'messages': stream.messages,
                    'bytes': stream.bytes
                }
                for stream in response.streams
            ]

        except Exception as e:
            return self.handle_error(e, "list streams") or []

    async def publish_to_stream(self, stream_name: str, subject: str, data: bytes) -> Optional[Dict]:
        """Publish message to JetStream stream."""
        try:
            await self._ensure_connected()
            request = nats_service_pb2.PublishToStreamRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                stream_name=stream_name,
                subject=subject,
                data=data
            )

            response = await self.stub.PublishToStream(request)

            if response.success:
                return {
                    'success': True,
                    'sequence': response.sequence
                }
            return None

        except Exception as e:
            return self.handle_error(e, "publish to stream")

    async def create_consumer(
        self,
        stream_name: str,
        consumer_name: str,
        filter_subject: str = ''
    ) -> Optional[Dict]:
        """
        Create JetStream consumer (pull-based, durable).

        Args:
            stream_name: Name of the stream to consume from
            consumer_name: Consumer name (should be lowercase-hyphens)
            filter_subject: Optional subject filter (lowercase.dots)

        Returns:
            Consumer info dict or None
        """
        try:
            await self._ensure_connected()

            config = nats_service_pb2.ConsumerConfig(
                name=consumer_name,
                durable_name=consumer_name,
                filter_subject=filter_subject,
                delivery_policy=nats_service_pb2.DELIVERY_ALL,
                ack_policy=nats_service_pb2.ACK_EXPLICIT
            )

            request = nats_service_pb2.CreateConsumerRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                stream_name=stream_name,
                config=config
            )

            response = await self.stub.CreateConsumer(request)

            if response.success:
                return {
                    'success': True,
                    'consumer': consumer_name
                }
            return None

        except Exception as e:
            return self.handle_error(e, "create consumer")

    async def pull_messages(self, stream_name: str, consumer_name: str, batch_size: int = 10) -> List[Dict]:
        """Pull messages from JetStream consumer."""
        try:
            await self._ensure_connected()

            request = nats_service_pb2.PullMessagesRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                stream_name=stream_name,
                consumer_name=consumer_name,
                batch_size=batch_size
            )

            response = await self.stub.PullMessages(request)

            return [
                {
                    'subject': msg.subject,
                    'data': msg.data,
                    'sequence': msg.sequence,
                    'num_delivered': msg.num_delivered
                }
                for msg in response.messages
            ]

        except Exception as e:
            return self.handle_error(e, "pull messages") or []

    async def ack_message(self, stream_name: str, consumer_name: str, sequence: int) -> Optional[Dict]:
        """Acknowledge message."""
        try:
            await self._ensure_connected()
            request = nats_service_pb2.AckMessageRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                stream_name=stream_name,
                consumer_name=consumer_name,
                sequence=sequence
            )

            response = await self.stub.AckMessage(request)

            if response.success:
                return {'success': True}
            return None

        except Exception as e:
            return self.handle_error(e, "ack message")

    # ============================================
    # KV Store Operations
    # ============================================

    async def kv_put(self, bucket: str, key: str, value: bytes) -> Optional[Dict]:
        """Put value in KV store."""
        try:
            await self._ensure_connected()

            request = nats_service_pb2.KVPutRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                bucket=bucket,
                key=key,
                value=value
            )

            response = await self.stub.KVPut(request)

            if response.success:
                return {
                    'success': True,
                    'revision': response.revision
                }
            return None

        except Exception as e:
            return self.handle_error(e, "kv put")

    async def kv_get(self, bucket: str, key: str) -> Optional[Dict]:
        """Get value from KV store."""
        try:
            await self._ensure_connected()
            request = nats_service_pb2.KVGetRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                bucket=bucket,
                key=key
            )

            response = await self.stub.KVGet(request)

            if response.found:
                return {
                    'found': True,
                    'value': response.value,
                    'revision': response.revision
                }
            return None

        except Exception as e:
            return self.handle_error(e, "kv get")

    async def kv_delete(self, bucket: str, key: str) -> Optional[Dict]:
        """Delete key from KV store."""
        try:
            await self._ensure_connected()
            request = nats_service_pb2.KVDeleteRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                bucket=bucket,
                key=key
            )

            response = await self.stub.KVDelete(request)

            if response.success:
                return {'success': True}
            return None

        except Exception as e:
            return self.handle_error(e, "kv delete")

    async def kv_keys(self, bucket: str) -> List[str]:
        """List all keys in KV bucket."""
        try:
            await self._ensure_connected()
            request = nats_service_pb2.KVKeysRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                bucket=bucket
            )

            response = await self.stub.KVKeys(request)
            return list(response.keys)

        except Exception as e:
            return self.handle_error(e, "kv keys") or []

    # ============================================
    # Object Store Operations
    # ============================================

    async def object_put(self, bucket: str, object_name: str, data: bytes) -> Optional[Dict]:
        """Put object in object store."""
        try:
            await self._ensure_connected()
            request = nats_service_pb2.ObjectPutRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                bucket=bucket,
                object_name=object_name,
                data=data
            )

            response = await self.stub.ObjectPut(request)

            if response.success:
                return {
                    'success': True,
                    'object_id': response.object_id
                }
            return None

        except Exception as e:
            return self.handle_error(e, "object put")

    async def object_get(self, bucket: str, object_name: str) -> Optional[Dict]:
        """Get object from object store."""
        try:
            await self._ensure_connected()
            request = nats_service_pb2.ObjectGetRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                bucket=bucket,
                object_name=object_name
            )

            response = await self.stub.ObjectGet(request)

            if response.found:
                return {
                    'found': True,
                    'data': response.data,
                    'metadata': dict(response.metadata)
                }
            return None

        except Exception as e:
            return self.handle_error(e, "object get")

    async def object_delete(self, bucket: str, object_name: str) -> Optional[Dict]:
        """Delete object from object store."""
        try:
            await self._ensure_connected()
            request = nats_service_pb2.ObjectDeleteRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                bucket=bucket,
                object_name=object_name
            )

            response = await self.stub.ObjectDelete(request)

            if response.success:
                return {'success': True}
            return None

        except Exception as e:
            return self.handle_error(e, "object delete")

    async def object_list(self, bucket: str) -> List[Dict]:
        """List objects in object store bucket."""
        try:
            await self._ensure_connected()
            request = nats_service_pb2.ObjectListRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                bucket=bucket
            )

            response = await self.stub.ObjectList(request)

            return [
                {
                    'name': obj.name,
                    'size': obj.size,
                    'metadata': dict(obj.metadata)
                }
                for obj in response.objects
            ]

        except Exception as e:
            return self.handle_error(e, "object list") or []

    # ============================================
    # Statistics
    # ============================================

    async def get_statistics(self) -> Optional[Dict]:
        """Get statistics."""
        try:
            await self._ensure_connected()
            request = nats_service_pb2.GetStatisticsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id
            )

            response = await self.stub.GetStatistics(request)

            return {
                'total_streams': response.total_streams,
                'total_consumers': response.total_consumers,
                'total_messages': response.total_messages,
                'total_bytes': response.total_bytes,
                'connections': response.connections,
                'in_msgs': response.in_msgs,
                'out_msgs': response.out_msgs
            }

        except Exception as e:
            return self.handle_error(e, "get statistics")

    # ============================================
    # Concurrent Operations
    # ============================================

    async def publish_many_concurrent(self, messages: List[Dict]) -> List[Optional[Dict]]:
        """
        Publish multiple messages concurrently.

        Args:
            messages: List of {'subject': str, 'data': bytes, 'headers': dict}

        Returns:
            List of publish results
        """
        import asyncio

        async def pub_single(msg: Dict) -> Optional[Dict]:
            return await self.publish(
                subject=msg['subject'],
                data=msg['data'],
                headers=msg.get('headers')
            )

        return await asyncio.gather(*[pub_single(m) for m in messages])


# Example usage
if __name__ == '__main__':
    import asyncio

    async def main():
        async with AsyncNATSClient(host='localhost', port=50056, user_id='test_user') as client:
            # Health check
            health = await client.health_check()
            print(f"Health: {health}")

            # Publish
            result = await client.publish('test.subject', b'Hello async NATS!')
            print(f"Publish: {result}")

            # Batch publish
            batch_result = await client.publish_batch([
                {'subject': 'test.1', 'data': b'msg1'},
                {'subject': 'test.2', 'data': b'msg2'},
                {'subject': 'test.3', 'data': b'msg3'}
            ])
            print(f"Batch publish: {batch_result}")

            # Stats
            stats = await client.get_statistics()
            print(f"Stats: {stats}")

    asyncio.run(main())



#!/usr/bin/env python3
"""
DuckDB gRPC Client
DuckDB OLAP 数据分析客户端
"""

from typing import List, Dict, Any, Optional, TYPE_CHECKING
from google.protobuf import struct_pb2
from google.protobuf.json_format import MessageToDict

from .base_client import BaseGRPCClient
from .proto import duckdb_service_pb2, duckdb_service_pb2_grpc, common_pb2

if TYPE_CHECKING:
    from .consul_client import ConsulRegistry


class DuckDBClient(BaseGRPCClient):
    """DuckDB gRPC 客户端"""

    def __init__(self, host: Optional[str] = None, port: Optional[int] = None, user_id: Optional[str] = None,
                 lazy_connect: bool = True, enable_compression: bool = True, enable_retry: bool = True,
                 consul_registry: Optional['ConsulRegistry'] = None, service_name_override: Optional[str] = None):
        """
        初始化 DuckDB 客户端

        Args:
            host: 服务地址 (optional, will use Consul discovery if not provided)
            port: 服务端口 (optional, will use Consul discovery if not provided)
            user_id: 用户 ID
            lazy_connect: 延迟连接 (默认: True)
            enable_compression: 启用压缩 (默认: True)
            enable_retry: 启用重试 (默认: True)
            consul_registry: ConsulRegistry instance for service discovery (optional)
            service_name_override: Override service name for Consul lookup (optional, defaults to 'duckdb')
        """
        super().__init__(
            host=host,
            port=port,
            user_id=user_id,
            lazy_connect=lazy_connect,
            enable_compression=enable_compression,
            enable_retry=enable_retry,
            consul_registry=consul_registry,
            service_name_override=service_name_override
        )
    
    def _create_stub(self):
        """创建 DuckDB service stub"""
        return duckdb_service_pb2_grpc.DuckDBServiceStub(self.channel)
    
    def service_name(self) -> str:
        return "DuckDB"

    def default_port(self) -> int:
        return 50052
    
    def _get_org_id(self) -> str:
        """获取组织ID"""
        return 'default_org'

    def get_table_prefix(self) -> str:
        """
        获取表名前缀

        DuckDB 服务使用平面命名约定：user_{user_id}_{table_name}

        Returns:
            表名前缀，例如: "user_test_user_"
        """
        return f"user_{self.user_id}_"

    def get_minio_bucket_name(self, base_bucket: str) -> str:
        """
        获取 MinIO 完整桶名（包含用户前缀）

        MinIO 服务为每个用户添加前缀：user-{sanitized_user_id}-{bucket}
        例如：user_id='test_user', bucket='duckdb-data' → 'user-test-user-duckdb-data'

        Args:
            base_bucket: 基础桶名

        Returns:
            完整的 MinIO 桶名
        """
        # Sanitize user_id: replace underscores with hyphens for DNS compliance
        sanitized_user_id = self.user_id.replace('_', '-')
        return f"user-{sanitized_user_id}-{base_bucket}"

    def qualify_table_name(self, table_name: str, use_prefix: bool = True) -> str:
        """
        为表名添加用户前缀

        Args:
            table_name: 表名
            use_prefix: 是否使用前缀

        Returns:
            带前缀的表名，例如: "user_test_user_hot_events"
        """
        if use_prefix and not table_name.startswith(self.get_table_prefix()):
            return f"{self.get_table_prefix()}{table_name}"
        return table_name

    def _qualify_sql_tables(self, sql: str) -> str:
        """
        在 SQL 语句中自动为表名添加用户前缀

        DuckDB 服务使用平面命名：user_{user_id}_{table_name}
        例如: "hot_events" -> "user_test_user_hot_events"

        支持的模式：
        - FROM table_name
        - JOIN table_name
        - INTO table_name
        - UPDATE table_name
        - TABLE table_name

        不会修改：
        - CTE (WITH ... AS) 定义的表名
        - 表别名
        - 系统表

        Args:
            sql: 原始 SQL 语句

        Returns:
            处理后的 SQL 语句
        """
        import re

        prefix = self.get_table_prefix()

        # Skip if SQL already contains prefixed tables for this user
        if prefix in sql:
            return sql

        # Extract CTE names from WITH clauses to exclude them from prefixing
        cte_names = set()
        cte_pattern = r'\bWITH\s+(\w+)\s+AS\s*\('
        for match in re.finditer(cte_pattern, sql, re.IGNORECASE):
            cte_names.add(match.group(1).lower())
        # Also match chained CTEs: WITH a AS (...), b AS (...)
        chained_cte_pattern = r',\s*(\w+)\s+AS\s*\('
        for match in re.finditer(chained_cte_pattern, sql, re.IGNORECASE):
            cte_names.add(match.group(1).lower())

        # Pattern to match table names in common SQL clauses
        patterns = [
            (r'\bFROM\s+([a-zA-Z_][a-zA-Z0-9_]*)\b', rf'FROM {prefix}\1'),
            (r'\bJOIN\s+([a-zA-Z_][a-zA-Z0-9_]*)\b', rf'JOIN {prefix}\1'),
            (r'\bINTO\s+([a-zA-Z_][a-zA-Z0-9_]*)\b', rf'INTO {prefix}\1'),
            (r'\bUPDATE\s+([a-zA-Z_][a-zA-Z0-9_]*)\b', rf'UPDATE {prefix}\1'),
            (r'\bTABLE\s+([a-zA-Z_][a-zA-Z0-9_]*)\b', rf'TABLE {prefix}\1'),
        ]

        result = sql
        for pattern, replacement in patterns:
            # Use negative lookahead to avoid matching system tables and CTEs
            safe_pattern = pattern.replace('([a-zA-Z_][a-zA-Z0-9_]*)',
                                          r'(?!(?:pg_|information_schema|sqlite_|duckdb_|temp\.|main\.))([a-zA-Z_][a-zA-Z0-9_]*)')

            # Apply replacement, but skip if table name is a CTE
            def replace_fn(match):
                table_name = match.group(1)
                if table_name.lower() in cte_names:
                    return match.group(0)  # Don't replace CTEs
                return match.group(0).replace(table_name, f"{prefix}{table_name}")

            result = re.sub(safe_pattern, replace_fn, result, flags=re.IGNORECASE)

        return result

    # ========================================
    # 数据库管理
    # ========================================
    
    def create_database(self, db_name: str, minio_bucket: str = '', metadata: Optional[Dict[str, str]] = None) -> Optional[Dict]:
        """
        创建数据库

        Args:
            db_name: 数据库名称
            minio_bucket: MinIO bucket for data storage
            metadata: Metadata dict

        Returns:
            数据库信息 {'database_id': str, 'database_name': str, ...} 或 None
        """
        try:
            self._ensure_connected()
            request = duckdb_service_pb2.CreateDatabaseRequest(
                database_name=db_name,
                user_id=self.user_id,
                organization_id=self._get_org_id(),
                minio_bucket=minio_bucket,
                metadata=metadata or {},
            )

            response = self.stub.CreateDatabase(request)

            if response.success:
                db_info = {
                    'database_id': response.database_info.database_id,
                    'database_name': response.database_info.database_name,
                    'minio_bucket': response.database_info.minio_bucket,
                    'size_bytes': response.database_info.size_bytes,
                    'table_count': response.database_info.table_count,
                }
                return db_info
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "创建数据库") or None
    
    def list_databases(self) -> List[Dict]:
        """
        列出所有数据库

        Returns:
            数据库列表
        """
        try:
            self._ensure_connected()
            request = duckdb_service_pb2.ListDatabasesRequest(
                user_id=self.user_id,
                organization_id=self._get_org_id(),
            )
            
            response = self.stub.ListDatabases(request)
            
            if response.success:
                databases = []
                for db in response.databases:
                    databases.append({
                        'database_id': db.database_id,
                        'name': db.database_name,
                        'size': db.size_bytes,
                        'table_count': db.table_count,
                        'created_at': str(db.created_at),
                    })
                return databases
            else:
                return []
                
        except Exception as e:
            return self.handle_error(e, "列出数据库") or []
    
    # ========================================
    # 查询操作
    # ========================================
    
    def execute_query(self, db_name: str, sql: str, limit: int = 100, auto_qualify_tables: bool = True) -> List[Dict]:
        """
        执行 SQL 查询

        Args:
            db_name: 数据库名称
            sql: SQL 查询语句
            limit: 返回结果限制
            auto_qualify_tables: 自动在 SQL 中添加 schema 前缀（默认: True）

        Returns:
            查询结果列表

        Note:
            如果 auto_qualify_tables=True，会自动在 SQL 中的表名前添加 user schema
            例如: "SELECT * FROM hot_events" -> "SELECT * FROM user_test_user.hot_events"
        """
        try:
            self._ensure_connected()

            # Auto-qualify table names in SQL if requested
            if auto_qualify_tables:
                sql = self._qualify_sql_tables(sql)

            request = duckdb_service_pb2.ExecuteQueryRequest(
                database_id=db_name,
                user_id=self.user_id,
                query=sql,
                max_rows=limit,
            )
            
            response = self.stub.ExecuteQuery(request)
            
            if response.success:
                # 转换结果
                # Row contains repeated Value (list), need to combine with column names
                columns = list(response.columns)
                results = []

                for row_msg in response.rows:
                    row_dict = {}
                    # row_msg.values is a repeated field (list of Value messages)
                    for i, value in enumerate(row_msg.values):
                        if i >= len(columns):
                            break
                        col_name = columns[i]

                        # Extract the actual value from the Value oneof field
                        if value.HasField('int_value'):
                            row_dict[col_name] = value.int_value
                        elif value.HasField('double_value'):
                            row_dict[col_name] = value.double_value
                        elif value.HasField('string_value'):
                            row_dict[col_name] = value.string_value
                        elif value.HasField('bool_value'):
                            row_dict[col_name] = value.bool_value
                        elif value.HasField('null_value'):
                            row_dict[col_name] = None
                        else:
                            row_dict[col_name] = None
                    results.append(row_dict)

                return results
            else:
                return []
                
        except Exception as e:
            return self.handle_error(e, "执行查询") or []
    
    def execute_statement(self, db_name: str, sql: str, auto_qualify_tables: bool = True) -> int:
        """
        执行写操作 (INSERT/UPDATE/DELETE)

        Args:
            db_name: 数据库名称
            sql: SQL 语句
            auto_qualify_tables: 自动在 SQL 中添加 schema 前缀（默认: True）

        Returns:
            影响的行数
        """
        try:
            self._ensure_connected()

            # Auto-qualify table names in SQL if requested
            if auto_qualify_tables:
                sql = self._qualify_sql_tables(sql)

            request = duckdb_service_pb2.ExecuteStatementRequest(
                database_id=db_name,
                user_id=self.user_id,
                statement=sql,
            )

            response = self.stub.ExecuteStatement(request)

            if response.success:
                return response.affected_rows
            else:
                return 0

        except Exception as e:
            return self.handle_error(e, "执行语句") or 0
    
    # ========================================
    # 表管理
    # ========================================
    
    def create_table(self, db_name: str, table_name: str, schema: Dict[str, str]) -> bool:
        """
        创建表

        Args:
            db_name: 数据库名称
            table_name: 表名
            schema: 列定义 {'column_name': 'data_type'}

        Returns:
            是否成功
        """
        try:
            self._ensure_connected()
            columns = [
                duckdb_service_pb2.ColumnInfo(
                    name=name,
                    data_type=dtype,
                )
                for name, dtype in schema.items()
            ]

            request = duckdb_service_pb2.CreateTableRequest(
                database_id=db_name,
                user_id=self.user_id,
                table_name=table_name,
                columns=columns,
            )

            response = self.stub.CreateTable(request)

            if response.success:
                return True
            else:
                return False

        except Exception as e:
            return self.handle_error(e, "创建表") or False
    
    def list_tables(self, db_name: str) -> List[str]:
        """
        列出所有表

        Args:
            db_name: 数据库名称

        Returns:
            表名列表
        """
        try:
            self._ensure_connected()
            request = duckdb_service_pb2.ListTablesRequest(
                database_id=db_name,
                user_id=self.user_id,
            )

            response = self.stub.ListTables(request)

            if response.success:
                tables = [table.table_name for table in response.tables]
                return tables
            else:
                return []

        except Exception as e:
            return self.handle_error(e, "列出表") or []
    
    # ========================================
    # 数据导入/导出
    # ========================================
    
    def import_from_minio(self, db_name: str, table_name: str,
                         bucket: str, object_key: str,
                         file_format: str = 'parquet') -> bool:
        """
        从 MinIO 导入数据

        Args:
            db_name: 数据库名称
            table_name: 目标表名
            bucket: MinIO 桶名
            object_key: 对象键
            file_format: 文件格式 (parquet/csv/json)

        Returns:
            是否成功
        """
        try:
            self._ensure_connected()
            request = duckdb_service_pb2.ImportFromMinIORequest(
                database_id=db_name,
                user_id=self.user_id,
                table_name=table_name,
                bucket_name=bucket,
                object_key=object_key,
                format=file_format,
            )

            response = self.stub.ImportFromMinIO(request)

            if response.success:
                return True
            else:
                return False

        except Exception as e:
            return self.handle_error(e, "从MinIO导入") or False
    
    def query_minio_file(self, db_name: str, bucket: str,
                        object_key: str, file_format: str = 'parquet',
                        limit: int = 100) -> List[Dict]:
        """
        直接查询 MinIO 中的文件（无需导入）

        Args:
            db_name: 数据库名称
            bucket: MinIO 桶名 (base name, server will add user prefix)
            object_key: 对象键
            file_format: 文件格式
            limit: 返回结果限制

        Returns:
            查询结果
        """
        try:
            self._ensure_connected()
            # Build query with $FILE placeholder (server will replace with full s3:// path)
            query = f"SELECT * FROM $FILE LIMIT {limit}"

            request = duckdb_service_pb2.QueryMinIOFileRequest(
                database_id=db_name,
                user_id=self.user_id,
                bucket_name=bucket,
                object_key=object_key,
                format=file_format,
                query=query,
            )

            response = self.stub.QueryMinIOFile(request)

            if response.success:
                # 转换结果
                # Row contains repeated Value (list), need to combine with column names
                columns = list(response.columns)
                results = []

                for row_msg in response.rows:
                    row_dict = {}
                    # row_msg.values is a repeated field (list of Value messages)
                    for i, value in enumerate(row_msg.values):
                        if i >= len(columns):
                            break
                        col_name = columns[i]

                        # Extract the actual value from the Value oneof field
                        if value.HasField('int_value'):
                            row_dict[col_name] = value.int_value
                        elif value.HasField('double_value'):
                            row_dict[col_name] = value.double_value
                        elif value.HasField('string_value'):
                            row_dict[col_name] = value.string_value
                        elif value.HasField('bool_value'):
                            row_dict[col_name] = value.bool_value
                        elif value.HasField('null_value'):
                            row_dict[col_name] = None
                        else:
                            row_dict[col_name] = None
                    results.append(row_dict)

                return results
            else:
                return []

        except Exception as e:
            return self.handle_error(e, "查询MinIO文件") or []
    
    def export_to_minio(self, db_name: str, query: str, bucket: str,
                       object_key: str, file_format: str = 'parquet',
                       overwrite: bool = True, auto_qualify_tables: bool = True) -> Optional[Dict]:
        """
        导出查询结果到 MinIO (Critical for hot-to-cold data flow)

        Args:
            db_name: 数据库名称
            query: SQL 查询（结果将导出）
            bucket: MinIO 桶名
            object_key: 对象键
            file_format: 文件格式 (parquet/csv/json, default: parquet)
            overwrite: 是否覆盖已存在的文件
            auto_qualify_tables: 自动在 SQL 中添加 schema 前缀（默认: True）

        Returns:
            导出结果信息 {'success': bool, 'rows_exported': int, 'file_size': int}
        """
        try:
            self._ensure_connected()

            # Auto-qualify table names in query if requested
            if auto_qualify_tables:
                query = self._qualify_sql_tables(query)

            request = duckdb_service_pb2.ExportToMinIORequest(
                database_id=db_name,
                user_id=self.user_id,
                query=query,
                bucket_name=bucket,
                object_key=object_key,
                format=file_format,
                overwrite=overwrite,
            )

            response = self.stub.ExportToMinIO(request)

            if response.success:
                return {
                    'success': True,
                    'rows_exported': response.rows_exported,
                    'file_size': response.file_size,
                    'execution_time_ms': response.execution_time_ms
                }
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "导出到MinIO") or None

    def execute_batch(self, db_name: str, statements: List[str],
                     use_transaction: bool = True, auto_qualify_tables: bool = True) -> Optional[Dict]:
        """
        批量执行 SQL 语句 (Critical for efficient hot data operations)

        Args:
            db_name: 数据库名称
            statements: SQL 语句列表
            use_transaction: 是否在事务中执行
            auto_qualify_tables: 自动在 SQL 中添加表名前缀（默认: True）

        Returns:
            执行结果 {'success': bool, 'results': List[Dict]}
        """
        try:
            self._ensure_connected()

            # Auto-qualify table names in all statements if requested
            if auto_qualify_tables:
                statements = [self._qualify_sql_tables(stmt) for stmt in statements]

            request = duckdb_service_pb2.ExecuteBatchRequest(
                database_id=db_name,
                user_id=self.user_id,
                statements=statements,
                transaction=use_transaction,
            )

            response = self.stub.ExecuteBatch(request)

            if response.success:
                results = []
                for r in response.results:
                    results.append({
                        'success': r.success,
                        'affected_rows': r.affected_rows,
                        'error': r.error if r.error else None
                    })
                return {
                    'success': True,
                    'results': results,
                    'total_execution_time_ms': response.total_execution_time_ms
                }
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "批量执行") or None

    def get_table_stats(self, db_name: str, table_name: str,
                       include_columns: bool = True) -> Optional[Dict]:
        """
        获取表统计信息 (Critical for understanding hot data characteristics)

        Args:
            db_name: 数据库名称
            table_name: 表名
            include_columns: 是否包含列统计

        Returns:
            统计信息 {'row_count': int, 'size_bytes': int, 'column_stats': List[Dict]}
        """
        try:
            self._ensure_connected()
            request = duckdb_service_pb2.GetTableStatsRequest(
                database_id=db_name,
                user_id=self.user_id,
                table_name=table_name,
                include_columns=include_columns,
            )

            response = self.stub.GetTableStats(request)

            if response.success:
                stats = {
                    'table_name': response.stats.table_name,
                    'row_count': response.stats.row_count,
                    'size_bytes': response.stats.size_bytes,
                    'column_stats': []
                }

                if include_columns:
                    for col_stat in response.stats.column_stats:
                        stats['column_stats'].append({
                            'column_name': col_stat.column_name,
                            'distinct_count': col_stat.distinct_count,
                            'null_count': col_stat.null_count,
                        })

                return stats
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "获取表统计") or None

    def delete_database(self, db_name: str, delete_from_minio: bool = True,
                       force: bool = False) -> bool:
        """
        删除数据库 (Critical for hot data lifecycle management)

        Args:
            db_name: 数据库名称
            delete_from_minio: 是否从 MinIO 删除文件
            force: 强制删除（忽略错误）

        Returns:
            是否成功
        """
        try:
            self._ensure_connected()
            request = duckdb_service_pb2.DeleteDatabaseRequest(
                database_id=db_name,
                user_id=self.user_id,
                delete_from_minio=delete_from_minio,
                force=force,
            )

            response = self.stub.DeleteDatabase(request)

            if response.success:
                return True
            else:
                return False

        except Exception as e:
            return self.handle_error(e, "删除数据库") or False

    def drop_table(self, db_name: str, table_name: str,
                  if_exists: bool = True, cascade: bool = False) -> bool:
        """
        删除表 (For managing hot data lifecycle)

        Args:
            db_name: 数据库名称
            table_name: 表名
            if_exists: 如果不存在不报错
            cascade: 级联删除依赖对象

        Returns:
            是否成功
        """
        try:
            self._ensure_connected()
            request = duckdb_service_pb2.DropTableRequest(
                database_id=db_name,
                user_id=self.user_id,
                table_name=table_name,
                if_exists=if_exists,
                cascade=cascade,
            )

            response = self.stub.DropTable(request)

            if response.success:
                return True
            else:
                return False

        except Exception as e:
            return self.handle_error(e, "删除表") or False

    def get_table_schema(self, db_name: str, table_name: str) -> Optional[Dict]:
        """
        获取表结构 (For introspection when selecting hot data)

        Args:
            db_name: 数据库名称
            table_name: 表名

        Returns:
            表结构信息 {'columns': List[Dict], 'row_count': int}
        """
        try:
            self._ensure_connected()
            request = duckdb_service_pb2.GetTableSchemaRequest(
                database_id=db_name,
                user_id=self.user_id,
                table_name=table_name,
            )

            response = self.stub.GetTableSchema(request)

            if response.success:
                columns = []
                for col in response.table_info.columns:
                    columns.append({
                        'name': col.name,
                        'data_type': col.data_type,
                        'nullable': col.nullable,
                        'is_primary_key': col.is_primary_key,
                    })

                schema = {
                    'table_name': response.table_info.table_name,
                    'columns': columns,
                    'row_count': response.table_info.row_count,
                    'size_bytes': response.table_info.size_bytes,
                }

                return schema
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "获取表结构") or None

    def get_database_info(self, db_name: str) -> Optional[Dict]:
        """
        获取数据库信息 (For monitoring hot data usage)

        Args:
            db_name: 数据库名称

        Returns:
            数据库信息 {'size_bytes': int, 'table_count': int, 'created_at': str}
        """
        try:
            self._ensure_connected()
            request = duckdb_service_pb2.GetDatabaseInfoRequest(
                database_id=db_name,
                user_id=self.user_id,
            )

            response = self.stub.GetDatabaseInfo(request)

            if response.success:
                info = {
                    'database_name': response.database_info.database_name,
                    'size_bytes': response.database_info.size_bytes,
                    'table_count': response.database_info.table_count,
                    'minio_bucket': response.database_info.minio_bucket,
                    'minio_path': response.database_info.minio_path,
                    'created_at': str(response.database_info.created_at),
                    'last_accessed': str(response.database_info.last_accessed),
                }

                return info
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "获取数据库信息") or None

    # ========================================
    # 健康检查
    # ========================================

    def health_check(self, detailed: bool = False) -> bool:
        """健康检查"""
        try:
            self._ensure_connected()
            request = duckdb_service_pb2.HealthCheckRequest(
                detailed=detailed,
            )
            
            response = self.stub.HealthCheck(request)

            if response.healthy:
                return True
            else:
                return False

        except Exception as e:
            return False


# 便捷使用示例
if __name__ == '__main__':
    # 使用 with 语句自动管理连接
    with DuckDBClient(host='localhost', port=50052, user_id='test_user') as client:
        # 健康检查
        client.health_check()
        
        # 数据库操作
        client.create_database('analytics', 'Analytics database')
        databases = client.list_databases()

        # 查询操作
        results = client.execute_query('analytics', 'SELECT * FROM users LIMIT 10')

